@@ -55,9 +55,10 @@ import org.xml.sax.helpers.DefaultHandler;
 
 /**
  * History:
- * Fix @1796: intermediate variables lost when put in a view
- * 20/06/11 @FIX 1806
+ * Fix 1796: intermediate variables lost when put in a view
+ * 20/06/11 FIX 1806
  * 16/03/13 Display number of match per file
+ * 04/10/13 FIX 2242 Error with overlap
  * @author osallou
  *
  */
@@ -200,16 +201,37 @@ public class LogolUtils {
 		return varSet;
 	}
 
-
+	/**
+	 * Gets all saved variable reference for the input data string
+	 * @param data
+	 * @param id current id of variable, -1 if not applicable
+	 * @param model  current model
+	 * @return a List of data reference
+	 */
+	public static String getAllSavedVariables(String data,int id,String model) {
+		return getSavedVariables(data, id, model, true);
+	}
 
 	/**
-	 * Gets the saved variable reference for the input data string
+	 * Gets the saved variable reference of the current predicate for the input data string
 	 * @param data
 	 * @param id current id of variable, -1 if not applicable
 	 * @param model  current model
 	 * @return a List of data reference
 	 */
 	public static String getSavedVariables(String data,int id,String model) {
+		return getSavedVariables(data, id, model, false);
+	}
+
+	/**
+	 * Gets the saved variable reference for the input data string
+	 * @param data
+	 * @param id current id of variable, -1 if not applicable
+	 * @param model  current model
+	 * @param all Get all referenced variables or only current predicate ones
+	 * @return a List of data reference
+	 */
+	public static String getSavedVariables(String data,int id,String model, boolean all) {
 		
 		String usedVars= "[";	
 		Vector<String> foundVariables=new Vector<String>();
@@ -230,6 +252,7 @@ public class LogolUtils {
 			}				
 		}	
 		
+		if(all) {
 		// Now add overlapped variables found in vector overlappedVariables
 		/* Overlapped variables need to be managed differently  because they are not saved
 		 * in current model, data references etc... are passed via parameters of called predicated.
@@ -237,12 +260,13 @@ public class LogolUtils {
 		 */
 		//View localView = new View();
 		//String model = localView.currentModel;
-		for(int i=0;i<LogolVariable.overlappedVariables.size();i++) {
-			VariableId lv_var = LogolVariable.overlappedVariables.get(i);
-			logger.debug("overlap var = "+lv_var.model+","+lv_var.name);
-			if(lv_var.model.equals(model)) {
-				if(!usedVars.equals("[")) { usedVars+=","; }
-				usedVars+=Constants.LOGOLVARREF+lv_var.name;
+			for(int i=0;i<LogolVariable.overlappedVariables.size();i++) {
+				VariableId lv_var = LogolVariable.overlappedVariables.get(i);
+				logger.debug("overlap var = "+lv_var.model+","+lv_var.name);
+				if(lv_var.model.equals(model)) {
+					if(!usedVars.equals("[")) { usedVars+=","; }
+						usedVars+=Constants.LOGOLVARREF+lv_var.name;
+					}
 			}
 		}
 		
