@@ -21,7 +21,7 @@ maxStart = ARGV[5]
 
 DIR=File.expand_path(File.dirname(__FILE__))
 
-if muteOnly == 1
+if muteOnly.to_i == 1
   cmd = "ruby -rubygems "+DIR+"/logolIndex.rb --search "+sequence+".index --min "+minStart+" --max "+maxStart+" --pattern "+motifFileName+" --error "+nbError+" --hamming"
 else
   cmd = "ruby -rubygems "+DIR+"/logolIndex.rb --search "+sequence+".index --min "+minStart+" --max "+maxStart+" --pattern "+motifFileName+" --error "+nbError
